"""This file contains all the Qualys API URLs used by qualyspy."""

about = "/msp/about.php"
host_list_detection = "/api/2.0/fo/asset/host/vm/detection/"
host_list = "/api/2.0/fo/asset/host/"
knowledgebase = "/api/2.0/fo/knowledge_base/vuln/"

gateway_auth = "/auth/"

asset_details = "/rest/2.0/get/am/asset"
all_asset_details = "/rest/2.0/search/am/asset"
